/******************************************************************
Copyright (c) 1996 Novell, Inc. All Rights Reserved.

THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND
TREATIES. USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE
LICENSE AGREEMENT ACCOMPANYING THE SOFTWARE DEVELOPMENT KIT (SDK)
THAT CONTAINS THIS WORK. PURSUANT TO THE SDK LICENSE AGREEMENT,
NOVELL HEREBY GRANTS TO DEVELOPER A ROYALTY-FREE, NON-EXCLUSIVE
LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS PRODUCT. NOVELL
GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF
DEVELOPER'S PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO
DEVELOPER OR DEVELOPER'S CUSTOMERS WITH RESPECT TO THIS CODE.
******************************************************************/

package com.novell.nds.dirxml.driver.jdbc.db;


import java.util.Collections;
import java.util.LinkedList;
import java.util.List;


/**
 * A system-independent time value.  This base class provides default
 * implementations for the non-system-specific methods of interface
 * <code>DBTimeTranslator</code>.
 *
 * @see DBTimeTranslator
 */
abstract public
class AbstractTimeValue
{

    static protected final Class DEFAULT_CLASS = java.sql.Timestamp.class;
    static protected final DSTimeFormat DEFAULT_IN_FORMAT = DSTimeFormat.DEFAULT;

    //NOTE:  SimpleDateFormat only works with 3 digit, millisecond scale
    //  (yyyyMMddHHmmssSSS); we need nano scale to 9 digits
    //  (yyyyMMddHHmmssSSSSSSSSS)
    static protected final int LEN_STRING_TIMESTAMP = 23; //CCYYMMDDHHMMSSNNNNNNNNN
    static protected final int LEN_STRING_DATE      = 8; //CCYYMMDD
    static protected final int LEN_STRING_TIME      = 6; //HHMMSS
    static protected final int MAX_SCALE_NANOS      = 9;
    static protected final int DS_LEN_TIMESTAMP_MIN = 14;
    static protected final String FORMAT_TIMESTAMP  = "yyyyMMddHHmmss";
    static protected final String FORMAT_DATE       = "yyyyMMdd";
    static protected final String FORMAT_TIME       = "HHmmss";


    protected String _time;
    protected Class _class;
    protected DSTimeFormat _outFormat;
    protected boolean _parseError = false;
    protected String _errorMessage = Constants.STR_EMPTY;
    protected List _exceptions = Collections.EMPTY_LIST;


    //prevents javadoc of default constructor
    protected
    AbstractTimeValue() {}


    protected
    void
    setOutFormat(String syntax)
    {
        if(DBLib.IsNotValue(syntax))
        {
            _outFormat = DEFAULT_IN_FORMAT;
        }
        else
        {
            syntax = syntax.trim();
            _outFormat = DSTimeFormat.GetFormat(syntax);

            if(_outFormat == null)
            {
                _outFormat = DEFAULT_IN_FORMAT;
            }
        }//else
    }//setOutFormat()


    protected
    void
    setClass(String c)
    {
        if(DBLib.IsNotValue(c))
        {
            _class = DEFAULT_CLASS;
        }
        else
        {
            c = c.trim();

            if(c.compareTo(DBTimeTranslator.CLASS_TIMESTAMP) == 0)
            {
                _class = java.sql.Timestamp.class;
            }
            else if(c.compareTo(DBTimeTranslator.CLASS_DATE) == 0)
            {
                _class = java.sql.Date.class;
            }
            else if(c.compareTo(DBTimeTranslator.CLASS_TIME) == 0)
            {
                _class = java.sql.Time.class;
            }
            else
            {
                c = c.toUpperCase();

                if(c.startsWith("TIMESTAMP"))
                {
                    _class = java.sql.Timestamp.class;
                }
                else if(c.startsWith("DATE"))
                {
                    _class = java.sql.Date.class;
                }
                else if(c.startsWith("TIME"))
                {
                    _class = java.sql.Time.class;
                }
                else
                {
                    _class = DEFAULT_CLASS;
                }
            }//else
        }//else
    }//setClass()


    protected
    void
    addException(Exception e)
    {
        if(_exceptions == Collections.EMPTY_LIST)
        {
            _exceptions = new LinkedList();
        }

        _exceptions.add(e);
    }//addException()


    /**
     * Returns any exceptions generated while parsing.  Returns a List of
     * <code>Exception</code>s with length > 0 if exceptions were generated;
     * <code>Collections&#46;EMPTY_LIST</code> otherwise.
     *
     * @return <code>true</code> if there was a parse error;
     * <code>false</code> otherwise; never returns <code>null</code>
     */
    public
    List
    getExceptions()
    {
        return _exceptions;
    }//getExceptions()


    /**
     * Indicates whether an error occurred parsing this timestamp/date/time
     * value.
     *
     * @return <code>true</code> if there was a parse error;
     * <code>false</code> otherwise
     */
    public
    boolean
    parseError()
    {
        return _parseError;
    }//parseError()


    /**
     * Returns the first parse error message, if any.
     *
     * @return a string with length > 0 if there was a parse error;
     * empty string otherwise; never returns <code>null</code>
     */
    public
    String
    getErrorMessage()
    {
        return _errorMessage;
    }//getErrorMessage()

}//class AbstractTimestamp
